#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.MainForm;
using HeuristicLab.PluginInfrastructure;
using System;
using System.Windows.Forms;

namespace HeuristicLab.Optimization.Networks.Views {
  [View("ClientPort View")]
  [Content(typeof(ClientPort), true)]
  [Content(typeof(IClientPort), false)]
  public partial class ClientPortView : ClientServicePortView {
    protected EntitySelectorDialog entitySelectorDialog;

    public new IClientPort Content {
      get { return (IClientPort)base.Content; }
      set { base.Content = value; }
    }

    public ClientPortView() {
      InitializeComponent();
      errorProvider.SetIconAlignment(servicePortView, ErrorIconAlignment.MiddleRight);
      errorProvider.SetIconPadding(servicePortView, 2);
    }

    protected override void Dispose(bool disposing) {
      if (disposing) {
        if (entitySelectorDialog != null) entitySelectorDialog.Dispose();
        if (components != null) components.Dispose();
      }
      base.Dispose(disposing);
    }

    protected override void DeregisterContentEvents() {
      Content.ServicePortChanged -= Content_ServicePortChanged;
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.ServicePortChanged += Content_ServicePortChanged;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      servicePortView.Content = Content == null ? null : Content.ServicePort;
      errorProvider.SetError(servicePortView, ((Content == null) || Content.Valid) ? string.Empty : "Port configurations do not match");
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      servicePortGroupBox.Enabled = Content != null && !ReadOnly;
      setServicePortButton.Enabled = Content != null && !ReadOnly;
      clearServicePortButton.Enabled = Content != null && Content.ServicePort != null && !ReadOnly;
      cloneServicePortParametersButton.Enabled = Content != null && Content.ServicePort != null && !ReadOnly;
    }

    protected virtual void Content_ServicePortChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_ServicePortChanged), sender, e);
      else {
        clearServicePortButton.Enabled = Content.ServicePort != null && !ReadOnly;
        cloneServicePortParametersButton.Enabled = Content.ServicePort != null && !ReadOnly;
        servicePortView.Content = Content.ServicePort;
      }
    }
    protected override void Content_InterfaceChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_InterfaceChanged), sender, e);
      else {
        base.Content_InterfaceChanged(sender, e);
        errorProvider.SetError(servicePortView, Content.Valid ? string.Empty : "Port configurations do not match");
      }
    }

    protected virtual void setServicePortButton_Click(object sender, EventArgs e) {
      if (entitySelectorDialog == null) {
        entitySelectorDialog = new EntitySelectorDialog();
        entitySelectorDialog.Caption = "Select Service Port";
        entitySelectorDialog.EntitySelector.Caption = "Available Service Ports";
      }

      try {
        IEntity root = Content;
        while (root.Parent != null)
          root = root.Parent;
        entitySelectorDialog.EntitySelector.Configure(
          root,
          Content.ServicePort,
          typeof(IServicePort)
        );

        if (entitySelectorDialog.ShowDialog(this) == DialogResult.OK) {
          Content.ServicePort = entitySelectorDialog.EntitySelector.SelectedEntity as IServicePort;
        }
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog(this, ex);
      }
    }
    protected virtual void clearServicePortButton_Click(object sender, EventArgs e) {
      Content.ServicePort = null;
    }

    protected virtual void servicePortView_DragEnterOver(object sender, DragEventArgs e) {
      e.Effect = DragDropEffects.None;
      var data = (IServicePort)e.Data.GetData(HeuristicLab.Common.Constants.DragDropDataFormat);

      if (!ReadOnly && (Content.IsValidServicePort(data))) {
        e.Effect = DragDropEffects.Link;
      }
    }
    protected virtual void servicePortView_DragDrop(object sender, DragEventArgs e) {
      if (e.Effect != DragDropEffects.None) {
        IServicePort port = e.Data.GetData(HeuristicLab.Common.Constants.DragDropDataFormat) as IServicePort;
        Content.ServicePort = port;
      }
    }
    protected virtual void cloneServicePortParametersButton_Click(object sender, EventArgs e) {
      Content.CloneServicePortParameters();
    }
  }
}
